# Generated from 'Fonts.h'

def FOUR_CHAR_CODE(x): return x
kNilOptions = 0
systemFont = 0
applFont = 1
kFMDefaultOptions = kNilOptions
kFMDefaultActivationContext = kFMDefaultOptions
kFMGlobalActivationContext = 0x00000001
kFMLocalActivationContext = kFMDefaultActivationContext
kFMDefaultIterationScope = kFMDefaultOptions
kFMGlobalIterationScope = 0x00000001
kFMLocalIterationScope = kFMDefaultIterationScope
kPlatformDefaultGuiFontID = applFont
kPlatformDefaultGuiFontID = -1
commandMark = 17
checkMark = 18
diamondMark = 19
appleMark = 20
propFont = 36864L
prpFntH = 36865L
prpFntW = 36866L
prpFntHW = 36867L
fixedFont = 45056L
fxdFntH = 45057L
fxdFntW = 45058L
fxdFntHW = 45059L
fontWid = 44208L
kFMUseGlobalScopeOption = 0x00000001
kFontIDNewYork = 2
kFontIDGeneva = 3
kFontIDMonaco = 4
kFontIDVenice = 5
kFontIDLondon = 6
kFontIDAthens = 7
kFontIDSanFrancisco = 8
kFontIDToronto = 9
kFontIDCairo = 11
kFontIDLosAngeles = 12
kFontIDTimes = 20
kFontIDHelvetica = 21
kFontIDCourier = 22
kFontIDSymbol = 23
kFontIDMobile = 24
newYork = kFontIDNewYork
geneva = kFontIDGeneva
monaco = kFontIDMonaco
venice = kFontIDVenice
london = kFontIDLondon
athens = kFontIDAthens
sanFran = kFontIDSanFrancisco
toronto = kFontIDToronto
cairo = kFontIDCairo
losAngeles = kFontIDLosAngeles
times = kFontIDTimes
helvetica = kFontIDHelvetica
courier = kFontIDCourier
symbol = kFontIDSymbol
mobile = kFontIDMobile
